@@ -335,7 +335,7 @@ static void dump_altsetting(struct usb_dev_handle *dev, struct usb_interface_des
 	char cls[128], subcls[128], proto[128];
 	char ifstr[128];
 	
-	char *buf;
+	unsigned char *buf;
 	unsigned size, i;
 
 	get_class_string(cls, sizeof(cls), interface->bInterfaceClass);
@@ -427,7 +427,7 @@ static void dump_endpoint(struct usb_dev_handle *dev, struct usb_interface_descr
 	static const char *syncattr[] = { "None", "Asynchronous", "Adaptive", "Synchronous" };
 	static const char *usage[] = { "Data", "Feedback", "Implicit feedback Data", "(reserved)" };
 	static const char *hb[] = { "1x", "2x", "3x", "(?\?)" };
-	char *buf;
+	unsigned char *buf;
 	unsigned size;
 
 	printf("      Endpoint Descriptor:\n"
@@ -1519,6 +1519,9 @@ dump_comm_descriptor(struct usb_dev_handle *dev, unsigned char *buf, char *inden
 		if (buf[3] & 0x01)
 			printf("%s    get/set/clear comm features\n", indent);
 		break;
+	// case 0x03:		/* direct line management */
+	// case 0x04:		/* telephone ringer */
+	// case 0x05:		/* telephone call and line state reporting */
 	case 0x06:		/* union desc */
 		if (buf [0] < 5)
 			goto bad;
@@ -1545,6 +1548,26 @@ dump_comm_descriptor(struct usb_dev_handle *dev, unsigned char *buf, char *inden
 				indent, buf[tmp], buf[tmp + 1]);
 		}
 		break;
+	case 0x08:		/* telephone operational modes */
+		if (buf [0] != 4)
+			goto bad;
+		printf("%sCDC Telephone operations:\n"
+		       "%s  bmCapabilities       0x%02x\n",
+		       indent,
+		       indent, buf[3]);
+		if (buf[3] & 0x04)
+			printf("%s    computer centric mode\n", indent);
+		if (buf[3] & 0x02)
+			printf("%s    standalone mode\n", indent);
+		if (buf[3] & 0x01)
+			printf("%s    simple mode\n", indent);
+		break;
+	// case 0x09:		/* USB terminal */
+	// case 0x0a:		/* network channel terminal */
+	// case 0x0b:		/* protocol unit */
+	// case 0x0c:		/* extension unit */
+	// case 0x0d:		/* multi-channel management */
+	// case 0x0e:		/* CAPI control management*/
 	case 0x0f:		/* ethernet functional desc */
 		if (buf [0] != 13)
 			goto bad;
@@ -1567,6 +1590,62 @@ dump_comm_descriptor(struct usb_dev_handle *dev, unsigned char *buf, char *inden
 		       indent, (buf[11]<<8)|buf[10],
 		       indent, buf[12]);
 		break;
+	// case 0x10:		/* ATM networking */
+	case 0x11:		/* WHCM functional desc */
+		if (buf[0] != 5)
+			goto bad;
+		printf("%sCDC WHCM:\n"
+		       "%s  bcdVersion           %x.%02x\n",
+		       indent,
+		       indent, buf[4], buf[3]);
+		break;
+	case 0x12:		/* MDLM functional desc */
+		if (buf [0] != 21)
+			goto bad;
+		printf("%sCDC MDLM:\n"
+		       "%s  bcdCDC               %x.%02x\n"
+		       "%s  bGUID               ",
+		       indent,
+		       indent, buf[4], buf[3],
+		       indent);
+		/* NOTE there are more traditional byteswap aware UUID-printing
+		 * schemes, something like X0X1X2X3-X4X5-X6X7-X8X9-XaXbXcXdXeXf
+		 * (or maybe it's X3X2X1X0-X5X4-...?) would be better
+		 */
+		dump_bytes(buf + 5, 16);
+		break;
+	case 0x13:		/* MDLM detail desc */
+		if (buf [0] < 5)
+			goto bad;
+		printf("%sCDC MDLM detail:\n"
+		       "%s  bGuidDescriptorType  %02x\n"
+		       "%s  bDetailData         ",
+		       indent,
+		       indent, buf[3],
+		       indent);
+		dump_bytes(buf + 4, buf[0] - 4);
+		break;
+	case 0x14:		/* device management functional desc */
+		if (buf[0] != 7)
+			goto bad;
+		printf("%sCDC Device Management:\n"
+		       "%s  bcdVersion           %x.%02x\n"
+		       "%s  wMaxCommand          %d\n",
+		       indent,
+		       indent, buf[4], buf[3],
+		       indent, (buf[6]<<8)| buf[5]);
+		break;
+	case 0x15:		/* OBEX functional desc */
+		if (buf[0] != 5)
+			goto bad;
+		printf("%sCDC OBEX:\n"
+		       "%s  bcdVersion           %x.%02x\n",
+		       indent,
+		       indent, buf[4], buf[3]);
+		break;
+	// case 0x16:		/* command set functional desc */
+	// case 0x17:		/* command set detail desc */
+	// case 0x18:		/* telephone control model functional desc */
 	default:
 		/* FIXME there are about a dozen more descriptor types */
 		printf("%sUNRECOGNIZED: ", indent);
@@ -1603,7 +1682,7 @@ static void do_hub(struct usb_dev_handle *fd, unsigned has_tt)
 
 	printf(" Hub Port Status:\n");
 	for (i = 0; i < buf[2]; i++) {
-		char stat [4];
+		unsigned char stat [4];
 
 		ret = usb_control_msg(fd,
 				USB_ENDPOINT_IN | USB_TYPE_CLASS
@@ -1803,7 +1882,7 @@ static void dumpdev(struct usb_device *dev)
 
 /* ---------------------------------------------------------------------- */
 
-static int dump_one_device(const char *path, unsigned int flags)
+static int dump_one_device(const char *path)
 {
 	struct usb_device *dev;
 	char vendor[128], product[128];
@@ -1823,7 +1902,7 @@ static int dump_one_device(const char *path, unsigned int flags)
 	return 0;
 }
 
-static int list_devices(int busnum, int devnum, int vendorid, int productid, unsigned int flags)
+static int list_devices(int busnum, int devnum, int vendorid, int productid)
 {
 	struct usb_bus *bus;
 	struct usb_device *dev;
@@ -1833,16 +1912,23 @@ static int list_devices(int busnum, int devnum, int vendorid, int productid, uns
 	status=1; /* 1 device not found, 0 device found */
 	
 	for (bus = usb_busses; bus; bus = bus->next) {
-		if (busnum != -1 && strtol(bus->dirname, NULL, 0) != busnum)
+		if (busnum != -1 && strtol(bus->dirname, NULL, 10) != busnum)
 			continue;
 		for (dev = bus->devices; dev; dev = dev->next) {
-			if (devnum != -1 && strtol(dev->filename, NULL, 0) != devnum)
+			if (devnum != -1 && strtol(dev->filename, NULL, 10)
+					!= devnum)
 				continue;
-			if ((vendorid != -1 && vendorid != dev->descriptor.idVendor) || (productid != -1 && productid != dev->descriptor.idProduct))
+			if ((vendorid != -1 && vendorid
+						!= dev->descriptor.idVendor)
+					|| (productid != -1 && productid
+						!= dev->descriptor.idProduct))
 				continue;
 			status = 0;
-			get_vendor_string(vendor, sizeof(vendor), dev->descriptor.idVendor);
-			get_product_string(product, sizeof(product), dev->descriptor.idVendor, dev->descriptor.idProduct);
+			get_vendor_string(vendor, sizeof(vendor),
+					dev->descriptor.idVendor);
+			get_product_string(product, sizeof(product),
+					dev->descriptor.idVendor,
+					dev->descriptor.idProduct);
 			if (verblevel > 0)
 				printf("\n");
 			printf("Bus %s Device %s: ID %04x:%04x %s %s\n",
@@ -1952,9 +2038,9 @@ int main(int argc, char *argv[])
 			}
 			*cp++ = 0;
 			if (*optarg)
-				vendor = strtoul(optarg, NULL, 0);
+				vendor = strtoul(optarg, NULL, 16);
 			if (*cp)
-				product = strtoul(cp, NULL, 0);
+				product = strtoul(cp, NULL, 16);
 			break;
 
 		case 'D':
@@ -1973,9 +2059,11 @@ int main(int argc, char *argv[])
 			"  -v, --verbose\n"
 			"      Increase verbosity (show descriptors)\n"
 			"  -s [[bus]:][devnum]\n"
-			"      Show only devices in specified bus and/or devnum\n"
+			"      Show only devices with specified device and/or\n"
+			"      bus numbers (in decimal)\n"
 			"  -d vendor:[product]\n"
-			"      Show only devices with the specified vendor and product ID\n"
+			"      Show only devices with the specified vendor and\n"
+			"      product ID numbers (in hexadecimal)\n"
 			"  -D device\n"
 			"      Selects which device lsusb will examine\n"
 			"  -t\n"
@@ -2008,8 +2096,8 @@ int main(int argc, char *argv[])
  		verblevel += 1 - VERBLEVEL_DEFAULT;
 		status = treedump();
 	} else if (devdump)
-		status = dump_one_device(devdump, 0);
+		status = dump_one_device(devdump);
 	else
-		status = list_devices(bus, devnum, vendor, product, 0);
+		status = list_devices(bus, devnum, vendor, product);
 	return status;
 }
